#pragma once

#include "../Common/Buffer.h"
#include "../CA/B25.h"
#include "TsHeader.h"
#include "TsStreamCallback.h"
#include "TsPsiSection.h"
#include "TsEpgMgr.h"
#include "TsDataMgr.h"
#include <map>
#include <set>

#define CONTINUITYCOUNTER_INIT	0xFF
#define TABLEID_INIT			0xFFFFFFFF

#define TS_OUTPUT_DEFAULT		0	// ftHgTsXg[o͔ԍ

#ifdef _DEBUG
//	#define TS_CONTINUITY_COUNTER_ERROR_LOG_ON		// Continuity Counter OȂȂRgAEg
#endif

struct TS_StreamInfo
{
	double	dSpeed;

	WORD	wTsPacketLen;
	DWORD	dwTsSyncFail;
	DWORD	dwTsPacketDrop;
	DWORD	dwScramblePacket;

	struct {
		DateTime stDateTime;
		DWORD dwGetTick;	// tA擾̃VXe
	} stCurrentTime;

	DWORD	dwRecieveBufferOverflow;
	DWORD	dwSyncBufferOverflow;
	DWORD	dwStreamBufferOverflow;
};

struct TS_PidInfo
{
	WORD type;						// PID̃^Cv
	BYTE old_continuity_counter;	// Ocontinuity_counter

	DWORD dwCurrentID;
	BOOL bStartFlg;					// JnĂ邩̃tO(ŏunit_start܂ł̓f[^Ȃ)
	CBuffer	*pBuf;					// PSIp̈ꎞf[^ۑpobt@

	BYTE byComponentTag;			// R|[lgʎq
	WORD wServiceID;				// T[rXID
};

struct TS_ServiceInfo
{
	char szServiceName[30];
};

typedef std::map<WORD,TS_ServiceInfo> TsServiceInfoMap;
typedef std::pair<WORD,TS_ServiceInfo> TsServiceInfoPair;

typedef std::set<WORD> TsFilterSet;
struct TS_Filter
{
	BOOL bRejectPATFlg;		// TRUEȂŵ̂ubNAFALSEȂŵ̈ȊOubN
	TsFilterSet PATFilter;	// w肷T[rXID(ubNPATUɂ肩)
	BOOL bRejectPIDFlg;
	TsFilterSet PIDFilter;	// w肷PID(ubNƃf[^͔Ȃ)
};
typedef std::map<WORD,struct TS_Filter> TsFilterMap;
typedef std::pair<WORD,struct TS_Filter> TsFilterPair;

// Error Log
#ifdef TS_CONTINUITY_COUNTER_ERROR_LOG_ON
	#define TS_CONTINUITY_COUNTER_ERROR_LOG_MAX	(1000)
	struct TS_CONTINUITY_COUNTER_ERROR_LOG
	{
		DateTime ErrorTime;
		WORD	wPID;
	};
#endif

class CTsStream
{
public:
	CTsStream(CB25 *pB25=NULL);
	~CTsStream(void);
	void ResetStream(BOOL bNewAlloc=TRUE);
	
	void SetDecodePSI(BOOL bDecode);
	void SetEpgProc(BOOL bProc);
	void SetNullPacketNoReject(BOOL bNoReject);
	void SetCallbackFunc(PF_TSENGINE_CALLBACK lpFunc,LPVOID lpContext);

	BOOL AddOutput(WORD wOutNo);
	BOOL DelOutput(WORD wOutNo);
	void ClearAllFilter(WORD wOutNo,BOOL bAllOutput=FALSE);
	void SetFilter(WORD wOutNo,BOOL bRejectPATFlg,WORD *pwServiceList,int iServiceNum,BOOL bRejectPIDFlg,WORD *pwPidList,int iPidNum);

	TsFilterMap m_Filter;

	void OnTsStream(BYTE *pStream,DWORD dwSize,TS_StreamInfo *lpstTsStreamInfo);
	void OnTsStreamTerminate(TS_StreamInfo *lpstTsStreamInfo);

	BOOL GetServiceName(WORD wServiceID,char *lpszBuf,int iBufLen);
	BOOL OutputDataContent(char *lpszDir,WORD wPID);

	// EpgMgr
	CTsEPGMgr* GetEpgMgr()
	{
		return &m_EpgMgr;
	}

	// f[^ϊ
	static void Convert_jpn_arib_char(char *str,size_t len,char *ret_buf,size_t ret_len);
	static DWORD Convert_bcd_to_dword(DWORD dwData);
protected:
	CBuffer *m_pBuf;
	CBuffer *m_pAlignment;
	BOOL m_bDecodePSI;
	BOOL m_bProcEpg;
	BOOL m_bNoRejectNullPacket;
	CB25 *m_pB25;
	PF_TSENGINE_CALLBACK *m_pCallbackFunc;
	LPVOID m_pCallbackContext;
	BYTE *m_pPayloadTemp;
	struct TS_PidInfo m_stPidTable[TS_PID_MAX];

	void DoCallback(UINT uType=TSENG_CALLBACK_NONE,WORD wStreamID=0,void *pData=NULL,DWORD dwVal=0)
	{
		if(m_pCallbackFunc)	m_pCallbackFunc(uType,wStreamID,(BYTE*)pData,dwVal,m_pCallbackContext);
	}

	void TsStreamStreamProc(BYTE *pAlignmentStream,DWORD dwAlignmentStreamLen,TS_StreamInfo *lpstTsStreamInfo);
	void MakeFakePAT(struct TS_Filter &stFilter,struct TS_header &stTsHeader,BYTE *pBuf,int pos,int len);

// PSIZNV֘A
	BOOL OnSection(struct TS_header &stTsHeader,struct TS_section_header &stSecHeader,BYTE *pData,size_t len,PF_SECTION_CALLBACK *lpCallback,LPVOID lpContext);
	int CheckSection(struct TS_header &stTsHeader,struct TS_section_header &stSecHeader,CBuffer *pBuffer,PF_SECTION_CALLBACK *lpCallback,LPVOID lpContext);

	static BOOL CALLBACK OnPAT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	static BOOL CALLBACK OnPMT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	static BOOL CALLBACK OnNIT_SIT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	static BOOL CALLBACK OnTOT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	static BOOL CALLBACK OnSDT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	static BOOL CALLBACK OnEIT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	// f[^
	static BOOL CALLBACK OnData(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	BOOL OnData_DII(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf);

	int OnTable(struct TS_header &stTsHeader,struct TS_section_header &stSecHeader,BYTE *pData,size_t &len,CBuffer **ppBuf);			

	// PSIZNVXV`FbNpL[̎擾
	// xxxxxxxx-yy-zz x=StreamID y=TableID z=SectionID
	inline DWORD MakePsiKey(WORD wStreamID,BYTE byTableID,BYTE bySectionID)
	{
		DWORD ret;
		ret = ((DWORD)wStreamID)<<16 | ((DWORD)byTableID)<<8 | bySectionID;
		return ret;
	};
	inline BOOL IsPsiUpdate(WORD wStreamID,BYTE byTableID,BYTE bySectionNo,BYTE byVersionNo)
	{
		DWORD dwPSIKey=MakePsiKey(wStreamID,byTableID,bySectionNo);
		std::map<DWORD,WORD>::iterator it=m_PsiVerNo.find(dwPSIKey);
		if(it==m_PsiVerNo.end())
		{
			// 
			m_PsiVerNo.insert(std::pair<DWORD,WORD>(dwPSIKey,MAKEWORD(byVersionNo,0)));
			return TRUE;
		} else {
			// XV
			if(it->second!=byVersionNo)
			{
				it->second=MAKEWORD(byVersionNo,0);
				return TRUE;
			}
		}
		return FALSE;
	};
	std::map<DWORD,WORD> m_PsiVerNo;

	// ݂̃Xg[
	struct TS_CurrentStreamInfo
	{
		// TOT
		WORD wMJDdate;
		DWORD dwBCDtime;

		// PAT
		WORD wStreamID;
	} m_stCurrentStream;	

	// PAT
	struct TS_PAT m_CurrentPAT;

	// EPGf[^x[X
	CTsEPGMgr m_EpgMgr;

	// f[^Ǘ
	CTsDataMgr m_DataMgr;

	// Service
	TsServiceInfoMap m_ServiceInfo;

#ifdef TS_CONTINUITY_COUNTER_ERROR_LOG_ON
	std::vector<TS_CONTINUITY_COUNTER_ERROR_LOG> m_CC_ErrorLog;
#endif
};
